@@ -339,17 +339,11 @@ class OPML(object):
 				})
 					
 		
-		name = os.path.join(os.path.dirname(path), 'outline.opml')
+		name = f'{path}.opml'
 		with open(name, 'w') as output:
 			xmltodict.unparse(opml, output, encoding='UTF8', pretty=True)
 		print(name)
 
-		if False:
-			name = os.path.join(os.path.dirname(path), 'outline.json')
-			with open(name, 'w') as output:
-				json.dump(opml, output, indent=4)
-			print(name)
-
 
 	#.............................................................
 	@args.operation
@@ -401,16 +395,58 @@ class OPML(object):
 			stack[-1]['outline'].append(outline)				
 
 			
-		name = os.path.join(os.path.dirname(path), 'outline.opml')
+		name = f'{path}.opml'
 		with open(name,'w') as output:
-			xmltodict.unparse(opml, output)
+			xmltodict.unparse(opml, output, encoding='UTF8', pretty=True)
 		print(name)
 
-		if False:
-			name = os.path.join(os.path.dirname(path), 'outline.json')
-			with open(name,'w') as output:
-				json.dump(opml, output, indent=4)
-			print(name)
+
+	#.............................................................
+	@args.operation
+	def xlsx2opml(self, file):
+		if not file.endswith('xlsx'):
+			sys.stderr.write('not an xlsx file\n')
+			return
+
+		path = os.path.expanduser(file)
+		workbook = open_workbook(filename=path)
+
+		opml = self._createOPML(file)
+		
+		body = opml['opml']['body']
+
+		# stack[-1] is the current parent
+		stack = [ body ]
+
+		def _push(stack, value):
+			if 'outline' not in stack[-1].keys():
+				stack[-1]['outline'] = []
+			outline = {
+				'@text': value,
+			}
+			stack[-1]['outline'].append(outline)				
+			parent = stack[-1]['outline'][-1]
+			stack.append(parent)
+				
+		def _pop(stack):
+			stack.pop()
+
+		for sheet in workbook.sheets():
+
+			_push(stack, sheet.name)
+
+			for r in range(sheet.nrows):
+			
+				for c in range(sheet.ncols):
+					_push(stack, sheet.cell(r,c).value)
+
+				for c in range(sheet.ncols):
+					_pop(stack)
+					
+		name = f'{path}.opml'
+		with open(name,'w') as output:
+			xmltodict.unparse(opml, output, encoding='UTF8', pretty=True)
+		print(name)
 
 
 #_________________________________________________________________
